package com.zanxiang.game.back.serve.pojo.dto;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 游戏回传策略表
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class GameBackPolicyDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    @NotBlank(message = "回传策略名称不能为空")
    @ApiModelProperty("回传策略名")
    private String backPolicyName;

    /**
     * 首充-小额判定金额（小于等于）
     */
    @NotNull
    @ApiModelProperty("首充-小额判定金额（小于等于）")
    private BigDecimal firstMinMoney;

    /**
     * 首充-小额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("首充-小额回传方式（概率/比例）")
    private Integer firstMinMoneyType;

    /**
     * 首充-小额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("首充-小额回传概率/比例")
    private String firstMinMoneyRate;

    /**
     * 首充-大额判定金额（大于等于）
     */
    @NotNull
    @ApiModelProperty("首充-大额判定金额（大于等于）")
    private BigDecimal firstMaxMoney;

    /**
     * 首充-大额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("首充-大额回传方式（概率/比例）")
    private Integer firstMaxMoneyType;

    /**
     * 首充-大额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("首充-大额回传概率/比例")
    private String firstMaxMoneyRate;

    /**
     * 首充-其它金额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("首充-其它金额回传方式（概率/比例）")
    private Integer firstOtherMoneyType;

    /**
     * 首充-其它金额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("首充-其它金额回传概率/比例")
    private String firstOtherMoneyRate;

    /**
     * 次单-小额判定金额（小于等于）
     */
    @NotNull
    @ApiModelProperty("次单-小额判定金额（小于等于）")
    private BigDecimal rechargeMinMoney;

    /**
     * 次单-小额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("次单-小额回传方式（概率/比例）")
    private Integer rechargeMinMoneyType;

    /**
     * 次单-小额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("次单-小额回传概率/比例")
    private String rechargeMinMoneyRate;

    /**
     * 次单-大额判定金额（大于等于）
     */
    @NotNull
    @ApiModelProperty("次单-大额判定金额（大于等于）")
    private BigDecimal rechargeMaxMoney;

    /**
     * 次单-大额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("次单-大额回传方式（概率/比例）")
    private Integer rechargeMaxMoneyType;

    /**
     * 次单-大额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("次单-大额回传概率/比例")
    private String rechargeMaxMoneyRate;

    /**
     * 次单-其它金额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("次单-其它金额回传方式（概率/比例）")
    private Integer rechargeOtherMoneyType;

    /**
     * 次单-其它金额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("次单-其它金额回传概率/比例")
    private String rechargeOtherMoneyRate;

    /**
     * 降档级别
     */
    @NotNull
    @ApiModelProperty("降档级别")
    private Integer levelDown;

    @ApiModelProperty("注册充值回传的最大间隔时间（超时不回传）")
    private Long regPayIntervalTime;


}
