package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.back.serve.dao.mapper.GameBackPolicyMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameBackPolicyDTO;
import com.zanxiang.game.back.serve.pojo.dto.GameBackPolicyListDTO;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.vo.GameBackPolicyVO;
import com.zanxiang.game.back.serve.service.IGameBackPolicyService;
import com.zanxiang.game.back.serve.service.IGameTencentBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentOrderService;
import com.zanxiang.game.back.serve.service.IGameTencentUserService;
import com.zanxiang.module.util.NumberUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.exception.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 游戏回传策略表 服务实现类
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Slf4j
@Service
public class GameBackPolicyServiceImpl extends ServiceImpl<GameBackPolicyMapper, GameBackPolicy> implements IGameBackPolicyService {
    @Autowired
    private IGameTencentBackLogService gameTencentBackLogService;
    @Autowired
    private IGameTencentOrderService gameTencentOrderService;
    @Autowired
    private IGameTencentUserService gameTencentUserService;
    @Autowired
    private IGameBackPolicyService gameBackPolicyService;
    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;

    @Override
    public IPage<GameBackPolicyVO> policyList(GameBackPolicyListDTO dto) {
        return gameBackPolicyService.page(dto.toPage(), new LambdaQueryWrapper<GameBackPolicy>()
                        .like(StringUtils.isNotBlank(dto.getBackPolicyName()), GameBackPolicy::getBackPolicyName, dto.getBackPolicyName())
                        .eq(GameBackPolicy::getCreateBy, SecurityUtil.getUserId())
                        .orderByDesc(GameBackPolicy::getCreateTime))
                .convert(this::toVO);
    }

    @Override
    public List<GameBackPolicyVO> allOfUser() {
        return list(new LambdaQueryWrapper<GameBackPolicy>()
                .eq(GameBackPolicy::getCreateBy, SecurityUtil.getUserId())
                .orderByDesc(GameBackPolicy::getCreateTime)
        ).stream().map(this::toVOSimple).collect(Collectors.toList());
    }

    @Override
    public boolean createPolicy(GameBackPolicyDTO dto) {
        if (nameExist(null, SecurityUtil.getUserId(), dto.getBackPolicyName())) {
            throw new BaseException("回传策略名重复");
        }
        GameBackPolicy backPolicy = GameBackPolicy.builder()
                .backPolicyName(dto.getBackPolicyName())
                .firstMinMoney(NumberUtil.multiply100(dto.getFirstMinMoney()).longValue())
                .firstMinMoneyType(dto.getFirstMinMoneyType())
                .firstMinMoneyRate(dto.getFirstMinMoneyRate())
                .firstMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .firstMaxMoneyType(dto.getFirstMaxMoneyType())
                .firstMaxMoneyRate(dto.getFirstMaxMoneyRate())
                .firstOtherMoneyType(dto.getFirstOtherMoneyType())
                .firstOtherMoneyRate(dto.getFirstOtherMoneyRate())
                .rechargeMinMoney(NumberUtil.multiply100(dto.getRechargeMinMoney()).longValue())
                .rechargeMinMoneyType(dto.getRechargeMinMoneyType())
                .rechargeMinMoneyRate(dto.getRechargeMinMoneyRate())
                .rechargeMaxMoney(NumberUtil.multiply100(dto.getRechargeMaxMoney()).longValue())
                .rechargeMaxMoneyType(dto.getRechargeMaxMoneyType())
                .rechargeMaxMoneyRate(dto.getRechargeMaxMoneyRate())
                .rechargeOtherMoneyType(dto.getRechargeOtherMoneyType())
                .rechargeOtherMoneyRate(dto.getRechargeOtherMoneyRate())
                .levelDown(dto.getLevelDown())
                .createBy(SecurityUtil.getUserId())
                .createTime(LocalDateTime.now())
                .regPayIntervalTime(dto.getRegPayIntervalTime())
                .build();
        return gameBackPolicyService.save(backPolicy);
    }

    @Override
    public boolean updatePolicy(Long policyId, GameBackPolicyDTO dto) {
        if (nameExist(policyId, SecurityUtil.getUserId(), dto.getBackPolicyName())) {
            throw new BaseException("回传策略名重复");
        }
        GameBackPolicy old = getById(policyId);
        GameBackPolicy backPolicy = GameBackPolicy.builder()
                .id(policyId)
                .backPolicyName(dto.getBackPolicyName())
                .firstMinMoney(NumberUtil.multiply100(dto.getFirstMinMoney()).longValue())
                .firstMinMoneyType(dto.getFirstMinMoneyType())
                .firstMinMoneyRate(dto.getFirstMinMoneyRate())
                .firstMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .firstMaxMoneyType(dto.getFirstMaxMoneyType())
                .firstMaxMoneyRate(dto.getFirstMaxMoneyRate())
                .firstOtherMoneyType(dto.getFirstOtherMoneyType())
                .firstOtherMoneyRate(dto.getFirstOtherMoneyRate())
                .rechargeMinMoney(NumberUtil.multiply100(dto.getRechargeMinMoney()).longValue())
                .rechargeMinMoneyType(dto.getRechargeMinMoneyType())
                .rechargeMinMoneyRate(dto.getRechargeMinMoneyRate())
                .rechargeMaxMoney(NumberUtil.multiply100(dto.getRechargeMaxMoney()).longValue())
                .rechargeMaxMoneyType(dto.getRechargeMaxMoneyType())
                .rechargeMaxMoneyRate(dto.getRechargeMaxMoneyRate())
                .rechargeOtherMoneyType(dto.getRechargeOtherMoneyType())
                .rechargeOtherMoneyRate(dto.getRechargeOtherMoneyRate())
                .levelDown(dto.getLevelDown())
                .regPayIntervalTime(dto.getRegPayIntervalTime())
                .createBy(old.getCreateBy())
                .createTime(old.getCreateTime())
                .updateBy(SecurityUtil.getUserId())
                .updateTime(LocalDateTime.now())
                .build();

        return gameBackPolicyService.updateById(backPolicy);
    }

    @Override
    public boolean deletePolicy(List<Long> ids) {
        return gameBackPolicyService.removeByIds(ids);
    }

    private boolean nameExist(Long policyId, Long userId, String policyName) {
        return count(new LambdaQueryWrapper<GameBackPolicy>()
                .ne(policyId != null, GameBackPolicy::getId, policyId)
                .eq(GameBackPolicy::getBackPolicyName, policyName)
                .eq(GameBackPolicy::getCreateBy, userId)
        ) > 0;
    }

    private GameBackPolicyVO toVOSimple(GameBackPolicy gameBackPolicy) {
        if (gameBackPolicy == null) {
            return null;
        }
        GameBackPolicyVO vo = BeanUtil.copy(gameBackPolicy, GameBackPolicyVO.class);
        vo.setFirstMinMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getFirstMinMoney())));
        vo.setFirstMaxMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getFirstMaxMoney())));
        vo.setRechargeMinMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getRechargeMinMoney())));
        vo.setRechargeMaxMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getRechargeMaxMoney())));
        return vo;
    }

    private GameBackPolicyVO toVO(GameBackPolicy gameBackPolicy) {
        if (gameBackPolicy == null) {
            return null;
        }
        GameBackPolicyVO vo = toVOSimple(gameBackPolicy);
        vo.setCreateName(sysUserRpc.getById(gameBackPolicy.getCreateBy()).getData().getNickname());
        return vo;
    }
}
