package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.tencent.base.pojo.dto.UserActionRpcDTO;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentMiniGameBackLogMapper;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameOrder;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameUser;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameTencentMiniGameBackLogService;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.JsonUtil;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Slf4j
@Service
public class GameTencentMiniGameBackLogServiceImpl extends ServiceImpl<GameTencentMiniGameBackLogMapper, GameTencentMiniGameBackLog>
        implements IGameTencentMiniGameBackLogService {
    @Autowired
    private RestTemplate restTemplate;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BackStatusEnum userBack(GameTencentMiniGameUser userLog) {
        GameTencentMiniGameBackLog backLog = GameTencentMiniGameBackLog.builder()
                .gameId(userLog.getGameId())
                .adAccountId(userLog.getAdAccountId())
                .clickId(userLog.getClickId())
                .actionTime(userLog.getRegisterTime())
                .wechatOpenid(userLog.getWechatOpenid())
                .wechatAppId(userLog.getWechatAppId())
                .actionType(ActionTypeEnum.REGISTER.getActionType())
                .createTime(LocalDateTime.now())
                .build();
        callback(backLog);
        save(backLog);
        return BackStatusEnum.getByValue(backLog.getBackStatus());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BackStatusEnum orderBack(GameTencentMiniGameOrder orderLog) {
        String actionType = orderLog.getOrderStatus().equals(OrderStatusEnum.SUCCESS_PAY.getValue()) ?
                ActionTypeEnum.PURCHASE.getActionType() : ActionTypeEnum.COMPLETE_ORDER.getActionType();
        GameTencentMiniGameBackLog backLog = GameTencentMiniGameBackLog.builder()
                .gameId(orderLog.getGameId())
                .adAccountId(orderLog.getAdAccountId())
                .clickId(orderLog.getClickId())
                .actionTime(orderLog.getOrderStatus().equals(OrderStatusEnum.SUCCESS_PAY.getValue()) ? orderLog.getPayTime() : orderLog.getRechargeTime())
                .wechatOpenid(orderLog.getWechatOpenid())
                .wechatAppId(orderLog.getWechatAppId())
                .actionType(actionType)
                .amount(orderLog.getRechargeMoney())
                .orderId(orderLog.getOrderId())
                .createTime(LocalDateTime.now())
                .build();
        callback(backLog);
        save(backLog);
        return BackStatusEnum.getByValue(backLog.getBackStatus());
    }

    private void callback(GameTencentMiniGameBackLog backLog) {
        Map<String, Object> actionParam = null;
        if (backLog.getAmount() != null) {
            actionParam = new HashMap<>(2);
            actionParam.put("value", backLog.getAmount());
        }
        TencentMiniGameBackRequest request = TencentMiniGameBackRequest.builder()
                .actions(Collections.singletonList(TencentMiniGameBackRequest.Action.builder()
                        .action_time(DateUtil.localDateTimeToSecond(backLog.getActionTime()))
                        .user_id(TencentMiniGameBackRequest.UserId.builder().wechat_app_id(backLog.getWechatAppId()).wechat_openid(backLog.getWechatOpenid()).build())
                        .trace(TencentMiniGameBackRequest.Trace.builder().click_id(backLog.getClickId()).build())
                        .account_id(backLog.getAdAccountId())
                        .action_set_id(null)
                        .action_type(backLog.getActionType())
                        .action_param(actionParam)
                        .build()))
                .build();
        try {
            log.error("腾讯小游戏回传参数：{}", JsonUtil.toString(request));
            ResponseEntity<Map> response = restTemplate.postForEntity("http://tracking.e.qq.com/conv", request, Map.class);
            log.error("腾讯小游戏回传完成：{}", JsonUtil.toString(response.getBody()));
            if (response.getStatusCode().is2xxSuccessful()) {
                if (Objects.equals(response.getBody().get("code").toString(), "0")) {
                    backLog.setBackStatus(BackStatusEnum.SUCCESS.getBackStatus());
                    return;
                }
            }
            backLog.setBackStatus(BackStatusEnum.FAILED.getBackStatus());
            backLog.setErrMsg(JsonUtil.toString(response.getBody()));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            backLog.setBackStatus(BackStatusEnum.FAILED.getBackStatus());
            backLog.setErrMsg(e.getMessage());
        }
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    public static class TencentMiniGameBackRequest {

        private List<Action> actions;

        @Data
        @NoArgsConstructor
        @AllArgsConstructor
        @Builder
        public static class Action {

            private Long action_time;

            private UserId user_id;
            private Trace trace;

            private Long account_id;

            private Long action_set_id;

            private String action_type;

            private Map<String, Object> action_param;
        }

        @Data
        @NoArgsConstructor
        @AllArgsConstructor
        @Builder
        public static class UserId {

            private String wechat_app_id;

            private String wechat_openid;

        }

        @Data
        @NoArgsConstructor
        @AllArgsConstructor
        @Builder
        public static class Trace {
            private String click_id;
        }
    }
}
