package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.pojo.dto.DataReportOfAppIdRpcDTO;
import com.zanxiang.advertising.tencent.base.pojo.dto.UserActionRpcDTO;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentOrderDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameBackPolicy;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentMiniGameOrder;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentOrder;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentOrderMapper;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentMiniGameOrderVO;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentOrderVO;
import com.zanxiang.game.back.serve.service.IGameBackPolicyService;
import com.zanxiang.game.back.serve.service.IGameTencentBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.utils.BackPolicyUtil;
import com.zanxiang.game.back.serve.utils.OrderUtil;
import com.zanxiang.game.module.base.ServerInfo;
import com.zanxiang.game.module.base.pojo.vo.AgentRpcVO;
import com.zanxiang.game.module.base.rpc.IAgentRpc;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 游戏腾讯订单表 服务实现类
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Slf4j
@Service
public class GameTencentOrderServiceImpl extends ServiceImpl<GameTencentOrderMapper, GameTencentOrder> implements IGameTencentOrderService {
    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;
    @Autowired
    private IGameTencentBackLogService gameTencentBackLogService;
    @Autowired
    private IGameBackPolicyService gameBackPolicyService;
    @DubboReference(providedBy = ServerInfo.SERVER_DUBBO_NAME)
    private IAgentRpc agentRpc;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean orderBack(GameTencentOrder orderLog, boolean mustBack) {
        boolean doBack = false;
        if (mustBack) {
            doBack = true;
        } else {
            if (!Objects.equals(orderLog.getOrderStatus(), OrderStatusEnum.SUCCESS_PAY.getValue())) {
                // 只要回传 支付行为
                return false;
            }
            if (orderLog.getBackPolicyId() == null) {
                // 没有回传策略，则直接全量回传
                doBack = true;
            } else {
                GameBackPolicy gameBackPolicy = gameBackPolicyService.getById(orderLog.getBackPolicyId());
                doBack = BackPolicyUtil.backOrder(orderLog.getOrderId(), gameBackPolicy, orderLog.getRechargeMoney(),
                        orderLog.getIsFirstOrder(), orderLog.getPayTime(), orderLog.getRegisterTime(),
                        (backUnit, isFirstOrder) -> Long.valueOf(list(new LambdaQueryWrapper<GameTencentOrder>()
                                        .eq(GameTencentOrder::getGameId, orderLog.getGameId())
                                        .eq(GameTencentOrder::getWechatAppId, orderLog.getWechatAppId())
                                        .eq(GameTencentOrder::getAdAccountId, orderLog.getAdAccountId())
                                        .eq(GameTencentOrder::getBackPolicyId, gameBackPolicy.getId())
                                        .eq(GameTencentOrder::getOrderStatus, OrderStatusEnum.SUCCESS_PAY.getValue())
                                        .eq(GameTencentOrder::getIsFirstOrder, isFirstOrder)
                                        .ne(GameTencentOrder::getOrderId, orderLog.getOrderId())
                                        .orderByDesc(GameTencentOrder::getCreateTime)
                                        .last("limit " + backUnit)
                                ).stream().filter(log -> log.getIsBack().equals(BackStatusEnum.SUCCESS.getBackStatus())).count()
                        ).intValue());
                orderLog.setRechargeMoney(BackPolicyUtil.lowRechargeLevel(orderLog.getRechargeMoney(), gameBackPolicy.getLevelDown()));
            }
        }
        if (doBack) {
            BackStatusEnum backStatus = doCallback(orderLog);
            update(new LambdaUpdateWrapper<GameTencentOrder>()
                    .set(GameTencentOrder::getIsBack, backStatus.getBackStatus())
                    .eq(GameTencentOrder::getId, orderLog.getId())
            );
            return true;
        }
        return false;
    }


    @Override
    public IPage<GameTencentOrderVO> tencentOrderLogList(GameTencentOrderDTO dto) {
        IPage<GameTencentOrder> page = page(dto.toPage(), new LambdaQueryWrapper<GameTencentOrder>()
                .eq(StringUtils.isNotBlank(dto.getChannel()), GameTencentOrder::getChannel, dto.getChannel())
                .eq(null != dto.getGameId(), GameTencentOrder::getGameId, dto.getGameId())
                .eq(null != dto.getAdAccountId(), GameTencentOrder::getAdAccountId, dto.getAdAccountId())
                .eq(StringUtils.isNotBlank(dto.getWechatAppId()), GameTencentOrder::getWechatAppId, dto.getWechatAppId())
                .eq(StringUtils.isNotBlank(dto.getWechatOpenid()), GameTencentOrder::getWechatOpenid, dto.getWechatOpenid())
                .eq(StringUtils.isNotBlank(dto.getOrderId()), GameTencentOrder::getOrderId, dto.getOrderId())
                .eq(null != dto.getIsBack(), GameTencentOrder::getIsBack, dto.getIsBack())
                .eq(null != dto.getOrderStatus(), GameTencentOrder::getOrderStatus, dto.getOrderStatus())
                .eq(dto.getIsFirstOrder() != null, GameTencentOrder::getIsFirstOrder, dto.getIsFirstOrder())
                .like(StringUtils.isNotBlank(dto.getRoleName()), GameTencentOrder::getRoleName, dto.getRoleName())
                .apply(dto.getRegPayIntervalTimeMin() != null, "if(pay_time is not null, TIMESTAMPDIFF(minute, register_time, pay_time), null) >= {0}", dto.getRegPayIntervalTimeMin())
                .apply(dto.getRegPayIntervalTimeMax() != null, "if(pay_time is not null, TIMESTAMPDIFF(minute, register_time, pay_time), null) <= {0}", dto.getRegPayIntervalTimeMax())
                .orderByDesc(GameTencentOrder::getCreateTime)
        );
        IPage<GameTencentOrderVO> result = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        if (CollectionUtils.isEmpty(page.getRecords())) {
            return result;
        }
        result.setRecords(toVOBatch(page.getRecords()));
        return result;
    }

    private List<GameTencentOrderVO> toVOBatch(List<GameTencentOrder> logList) {
        if (CollectionUtils.isEmpty(logList)) {
            return Collections.emptyList();
        }
        Set<String> agentKeys = logList.stream().map(GameTencentOrder::getChannel).filter(StringUtils::isNotBlank).collect(Collectors.toSet());
        Map<String, AgentRpcVO> agentMap = CollectionUtils.isEmpty(agentKeys) ? Collections.emptyMap() : agentRpc.getByAgentKeys(new ArrayList<>(agentKeys))
                .getData().stream().collect(Collectors.toMap(AgentRpcVO::getAgentKey, Function.identity()));
        return logList.stream().map(log -> {
            GameTencentOrderVO vo = toVOSimple(log);
            if (StringUtils.isNotBlank(log.getChannel())) {
                AgentRpcVO agent = agentMap.get(log.getChannel());
                if (agent != null) {
                    vo.setAgentName(agent.getAgentName());
                }
            }
            return vo;
        }).collect(Collectors.toList());
    }

    private GameTencentOrderVO toVOSimple(GameTencentOrder log) {
        if (log == null) {
            return null;
        }
        GameTencentOrderVO vo = BeanUtil.copy(log, GameTencentOrderVO.class);
        vo.setRegPayIntervalTime(OrderUtil.regPayIntervalTime(log.getPayTime(), log.getRegisterTime()));
        return vo;
    }

    @Override
    public boolean tencentOrderReport(List<Long> ids) {
        listByIds(ids).forEach(orderLog -> {
            BackStatusEnum backStatus = doCallback(orderLog);
            update(new LambdaUpdateWrapper<GameTencentOrder>()
                    .set(GameTencentOrder::getIsBack, backStatus.getBackStatus())
                    .eq(GameTencentOrder::getId, orderLog.getId())
            );
        });
        return true;
    }

    private BackStatusEnum doCallback(GameTencentOrder orderLog) {
        String actionType = orderLog.getOrderStatus().equals(OrderStatusEnum.SUCCESS_PAY.getValue()) ?
                ActionTypeEnum.PURCHASE.getActionType() : ActionTypeEnum.COMPLETE_ORDER.getActionType();
        Map<String, Object> actionParam = new HashMap<>(2);
        actionParam.put("claim_type", 0);
        actionParam.put("value", orderLog.getRechargeMoney());
        DataReportOfAppIdRpcDTO dataReportOfAppIdRpcDTO = DataReportOfAppIdRpcDTO.builder()
                .appId(orderLog.getWechatAppId())
                .userActionSetId(orderLog.getUserActionSetId())
                .action(UserActionRpcDTO.builder()
                        .actionTime(orderLog.getOrderStatus().equals(OrderStatusEnum.SUCCESS_PAY.getValue()) ? orderLog.getPayTime() : orderLog.getRechargeTime())
                        .actionType(actionType)
                        .userId(UserActionRpcDTO.UserIdRpcDTO.builder()
                                .wechatAppId(orderLog.getWechatAppId())
                                .wechatOpenid(orderLog.getWechatOpenid())
                                .build())
                        .actionParam(actionParam)
                        .build())
                .build();

        GameTencentBackLog gameTencentBackLog = GameTencentBackLog.builder()
                .gameId(orderLog.getGameId())
                .adAccountId(orderLog.getAdAccountId())
                .actionTime(orderLog.getOrderStatus().equals(OrderStatusEnum.SUCCESS_PAY.getValue()) ? orderLog.getPayTime() : orderLog.getRechargeTime())
                .createTime(LocalDateTime.now())
                .actionType(actionType)
                .orderId(orderLog.getOrderId())
                .wechatAppId(orderLog.getWechatAppId())
                .wechatOpenid(orderLog.getWechatOpenid())
                .userActionSetId(orderLog.getUserActionSetId())
                .actionParam(actionParam.toString())
                .build();
        ResultVO<Boolean> result = userActionSetRpc.reportByAppId(dataReportOfAppIdRpcDTO);
        BackStatusEnum backStatus = BackStatusEnum.FAILED;
        if (result.isSuccess()) {
            backStatus = BackStatusEnum.SUCCESS;
            gameTencentBackLog.setBackLog("回传成功");
        } else {
            log.error("回传腾讯订单失败，失败原因：{}", result.getMsg());
            gameTencentBackLog.setBackLog(result.getMsg());
        }
        gameTencentBackLogService.save(gameTencentBackLog);
        return backStatus;
    }
}
