package com.zanxiang.game.data.serve.pojo.entity;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.nutz.dao.entity.annotation.Column;
import org.nutz.dao.entity.annotation.PK;
import org.nutz.dao.entity.annotation.Table;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;

/**
 * <p>
 * 整体概括
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Table(AdsOverallSummary.TABLE_NAME)
@PK({"dt", "agentId", "accountId", "sourceSystem"})
public class AdsOverallSummary implements Serializable {
    private static final long serialVersionUID = 1L;
    public static final String TABLE_NAME = "ads_overall_summary";

    /**
     * 日期（用户注册时间）
     */
    private LocalDate dt;

    /**
     * 渠道ID
     */
    private Long agentId;

    /**
     * 广告账号
     */
    private Long accountId;

    /**
     * SDK来源
     */
    private String sourceSystem;

    /**
     * 投手ID
     */
    @Column
    private Long pitcherId;

    /**
     * 游戏ID
     */
    @Column
    private Long gameId;

    /**
     * 今日新增玩家数量
     */
    @Column
    private Long todayPlayerCount;

    /**
     * 昨日新增玩家数量
     */
    @Column
    private Long yesterdayPlayerCount;

    /**
     * 今日消耗
     */
    @Column
    private BigDecimal todayTotalCost;

    /**
     * 昨日消耗
     */
    @Column
    private BigDecimal yesterdayTotalCost;

    /**
     * 首日新用户充值
     */
    @Column
    private BigDecimal firstNewUserAmount;

    /**
     * 今日新用户充值
     */
    @Column
    private BigDecimal todayNewPlayerAmount;

    /**
     * 今日账面充值
     */
    @Column
    private BigDecimal todayAmount;

    /**
     * 今日账面充值（买量）
     */
    @Column
    private BigDecimal todayBuyAmount;

    /**
     * 今日账面充值（自然量）
     */
    @Column
    private BigDecimal todayNatureAmount;

    /**
     * 昨日新用户充值
     */
    @Column
    private BigDecimal yesterdayNewPlayerAmount;

    /**
     * 昨日账面充值
     */
    @Column
    private BigDecimal yesterdayAmount;

    /**
     * 今日新增渠道数量
     */
    @Column
    private Long todayAgentCount;

    /**
     * 昨日新增渠道数量
     */
    @Column
    private Long yesterdayAgentCount;

    /**
     * 今日新增游戏数量
     */
    @Column
    private Long todayGameCount;

    /**
     * 昨日新增游戏数量
     */
    @Column
    private Long yesterdayGameCount;

    /**
     * 首日回本率
     */
    @Column
    private BigDecimal firstRoi;

    /**
     * 7日回本率
     */
    @Column
    private BigDecimal d7TotalRoi;

    /**
     * 30日回本率
     */
    @Column
    private BigDecimal d30TotalRoi;

    /**
     * 总回本率
     */
    @Column
    private BigDecimal totalRoi;

}
